/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2012                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/

#include <vcg/complex/complex.h>
#include <wrap/io_trimesh/import_ply.h>
#include <wrap/io_trimesh/export_ply.h>
#include <vcg/complex/algorithms/update/color.h>
#include <vcg/complex/algorithms/geodesic.h>
#include <algorithm>
using namespace vcg;
using namespace std;


class MyEdge;
class MyFace;
class MyVertex;
struct MyUsedTypes : public vcg::UsedTypes<	vcg::Use<MyVertex>   ::AsVertexType,
                                        vcg::Use<MyEdge>     ::AsEdgeType,
                                        vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex<MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::VFAdj, vertex::Qualityf, vcg::vertex::Color4b, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face<  MyUsedTypes, vcg::face::VFAdj,  vcg::face::VertexRef, vcg::face::FFAdj, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<  MyUsedTypes, vcg::edge::VertexRef, vcg::edge::EFAdj, vcg::edge::BitFlags>{};
class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};

class GeodesicPos
{
public:
  face::Pos<MyFace> p;
  float dist;

  bool operator < (const GeodesicPos & p) const
  {
    return dist > p.dist;
  }

  GeodesicPos(MyFace *f, int edgeInd)
  {
    p = face::Pos<MyFace>(f,edgeInd);
    face::Pos<MyFace> tp= p;
    tp.FlipF();
    tp.FlipE();
    tp.FlipV();
    dist = tp.V()->Q();
  }
};

int main()
{
  MyMesh m;
  tri::io::ImporterPLY<MyMesh>::Open(m,"2handle.ply");

  vector<MyMesh::VertexPointer> seedVec;
  MyMesh::FacePointer startFace=&m.face[0];
  seedVec.push_back(startFace->V(0));
  seedVec.push_back(startFace->V(1));
  seedVec.push_back(startFace->V(2));
  tri::UpdateTopology<MyMesh>::VertexFace(m);
  tri::Geodesic<MyMesh>::Compute(m,seedVec);
  tri::UpdateColor<MyMesh>::PerVertexQualityRamp(m);
  tri::io::ExporterPLY<MyMesh>::Save(m,"out.ply",tri::io::Mask::IOM_VERTCOLOR|tri::io::Mask::IOM_VERTQUALITY);
  tri::UpdateTopology<MyMesh>::FaceFace(m);

  vector<GeodesicPos> heap;
  heap.push_back(GeodesicPos(startFace,0));
  heap.push_back(GeodesicPos(startFace,1));
  heap.push_back(GeodesicPos(startFace,2));

  make_heap(heap.begin(),heap.end());

  tri::UpdateSelection<MyMesh>::FaceClear(m);
  startFace->SetS();
  int cnt=0;
  while(!heap.empty())
  {
    pop_heap(heap.begin(),heap.end());
    GeodesicPos cur = heap.back();
    heap.pop_back();
    MyFace *curFace = cur.p.FFlip();
    if(!curFace->IsS())
    {
      curFace->SetS();
      for(int i=0;i<3;++i)
      {
        if(!curFace->FFp(i)->IsS())
          heap.push_back(GeodesicPos(curFace,i));
      }
      make_heap(heap.begin(),heap.end());
    }
    if(cnt++%100 == 0)
    {
      MyMesh tmp;
      printf("Saving snapshot...\n");
      tri::Append<MyMesh,MyMesh>::Selected(tmp,m);
      char buf[128]; sprintf(buf,"out%03i.ply",cnt);
      tri::io::ExporterPLY<MyMesh>::Save(tmp,buf,tri::io::Mask::IOM_VERTCOLOR|tri::io::Mask::IOM_VERTQUALITY);
    }
  }
  return 0;
}
